
#ifndef  HecClusterBuildAlg_H
#define HecClusterBuildAlg_H

#include "HecRunHeader2001v0.h"
#include "HecEvent.h"
#include "Geometry.h"
#include "TH1.h"
#include "TCanvas.h"
#include "TF1.h"
#include "SystemAlg.h"


/*! -Class:    HecClusterBuildAlg
  - Author:   Ian Gable (igable@uvic.ca)
  - Date:      06/04/2002
  - Description: HecClusterBuildAlg is a simple class which will construct a histogram
                          from a set of cells. Currently the list of cells is hardcoded. Later it could
                          be a vector of cells passed via the constructor.
  */
class HecClusterBuildAlg  :  public SystemAlg {
public:

  /** The user would call this constructor from inside the constructor of the
    *   UserAlg class.
    * @param event A pointer in the event. Information contained in the event
    *                             gets refreshed before the execute method is run
    * @param hecGeom Pointer to the singleton Geometry object
    * @param runHeader Pointer to the singleton HecRunHeader2001v0 object
    */

  HecClusterBuildAlg(Event* event, Geometry* hecGeom, Headers* runHeader);

  ~HecClusterBuildAlg();

  //! Gets run once at the begining of the event Loop
  bool initialize();

  //! Gets run for every event
  bool execute();

  //! Gets run after all events have been executed
  bool finalize();

  void setInteractive(bool interactive){ m_interactive = interactive; };

  bool isInteractive(){ return m_interactive; };

private:

  //! Private method of symmetric fit about the mean with a fixed sigma range
  void symmetricFit();

  Geometry* m_geo;
  Event* m_event;
  Headers* m_runHeader;

  double sigma, mean, sigma2, mean2;
  float xmin, xmax;  
  int j;
  TF1 *func1, *func2;

  TH1F* energy;
  int m_cut;
  int* m_hCollision;
  bool m_interactive;
  void printTrigger();

};

#endif
