
#ifndef  PlotAllCellsAlg_H
#define PlotAllCellsAlg_H

#include "HecRunHeader2001v0.h"
#include "HecEvent.h"
#include "Geometry.h"
#include "TH1.h"
#include "TCanvas.h"
#include "SystemAlg.h"


/*! -Class:   PlotAllCellsAlg
  - Author:   Ian Gable (igable@uvic.ca) with small adjustments by 
              Tamara Starke (tmstarke@uvic.ca)
  - Date:     05/10/2002
  - Description: PlotAllCellsAlg will create in the currnt working 
                 directory a ps file containing a histogram
                 for every single cells in the Hec.
  - Warning: This algorithm will create MORE THEN 300 ps files.
  */
class PlotAllCellsAlg : public SystemAlg {
public:

  /** The user should never need to use this contructor
    * @param event A pointer in the event. Information contained in 
                   the event gets refreshed before the execute 
		   method is run
    * @param hecGeom Pointer to the singleton HecGeometry object
    * @param runHeader Pointer to the singleton HecRunHeader object
    */

  PlotAllCellsAlg(Event* event, Geometry* hecGeom, Headers* runHeader);

  //! Gets run once at the begining of the event Loop
  bool initialize();

  //! Gets run for every event
  bool execute();

  //! Gets run after all events have been executed
  bool finalize();

  struct ADCMean {
    int cell;
    float mean;
  };
  
  vector<ADCMean*> cell_means; 

  class MeanSort {
  public:
    bool operator()(const ADCMean& set1, const ADCMean& set2) const{
      return set2.mean < set1.mean;
    }
  };

private:
  Geometry* m_geo;
  Event* m_event;
  Headers* m_runHeader;



  vector<TH1F*> energy;
  int m_cut;

  //################################
  //### insert your member variables here

  int num_entries[1200];
  float sum[1200];

  //################################
};


class HistoHighMean {
 public:
  bool operator()(const TH1F& hist1, const TH1F& hist2) const{
    return hist2.GetMean() < hist1.GetMean();
  }
};



#endif


