
#include "HecClusterBuildAlg.h"
#include "TROOT.h"
#include "TStyle.h"

#include <sstream>
#include <string>
#include <iostream>

using std::cout;
using std::endl;
using std::cin;
using std::vector;
using std::string;

HecClusterBuildAlg::HecClusterBuildAlg(Event* event, Geometry* geo, Headers* runHeader)
  :SystemAlg(event,geo,runHeader){
  m_event = event;
  m_geo = geo;
  m_runHeader = runHeader;
  m_cut = -5;
  m_interactive = true;

  // eventually could be passed via a vector
  m_hCollision = new int[32];
  int  temp[32] =  {61,41,37,33,13,
                     90,86,82,66,62,58,42,38,34,
                      115,111,107,91,87,83,67,63,59,
                    140,136,132,116,112,108,92,88,84} ;

  for( int i =0; i < 32; i++){
    m_hCollision[i] = temp[i];
  }
}


// initialize any variables here
bool HecClusterBuildAlg::initialize(){

  cout << "HecClusterBuildAlg Initialize()" << endl;
  string runNumber(" Run Number: " );
  std::ostringstream os;

  os  << runNumber << " " << m_runHeader->global()->getRunNo();
  runNumber= os.str();

  energy= new TH1F("energy", runNumber.c_str() ,(int)(250-(m_cut/3.2)) ,m_cut,800);

 return true;

}

// put any code you want executed per event here
bool HecClusterBuildAlg::execute(){

  float clusterSum = 0;

  for(int i = 0; i < 32; i++){
    if(m_event->hecCell(m_hCollision[i])->signal()>m_cut )  {
      clusterSum += m_event->hecCell(m_hCollision[i])->signal();
    }
  }
  energy->Fill(clusterSum);

  return true;
}

//Wrap up anything you wanted done after all the events have been run
// ie Draw your histograms etc
bool HecClusterBuildAlg::finalize(){

  TCanvas can("Energy", "Energy Canvas",600,800);
  //energy->Fit("gaus");

  gStyle->SetOptStat(1111);
  gStyle->SetStatW(0.15);
  gStyle->SetStatH(0.08);
  gStyle->SetStatBorderSize(0);
  gStyle->SetTitleBorderSize(0);
  gStyle->SetOptFit(111);

  energy->Draw();
  //symmetricFit();   // Calls the routine for the fit..  //05/06/2003 causes a segfault on the fate macines attention Tayfun

  can.Update();


  if(m_interactive){
    string choice;
    cout << "Output plot to postscript file and root file? (y/n): \n";
    //cin >> choice;
    choice = 'n';

    if(choice == "y"){
      can.Print("HecClusterBuild.ps") ;
      cout << "\nFile output to 'HecClusterBuild.ps'\n";
    } else {
      cout << "Finished HecClusterBuildAlg!\n";
    }
  }

  std::ostringstream os;
  os << m_runHeader->global()->getRunNo() << "_HecCluster.root";

  TFile file(os.str().c_str(), "recreate");
  energy->Write();
  file.Close();



  delete energy;

  return true;

}


void HecClusterBuildAlg::symmetricFit(){
   //#######################################
  // Routine which finds the best region for the fit
  // by iterating mean with fixed 4*sigma region

  // 05/06/2003: this function causes a segfault on the fates.
  
  cout << "\n Beginning iteration... \n" << endl; 

  func1 = new TF1("func1", "gaus", m_cut, 800);
  energy->Fit("func1","RQ0"); 
  mean = func1->GetParameter(1);
  sigma = func1->GetParameter(2); 
     
   for(j=0;j<10;j++) {
   xmin = mean - 2.0*sigma;   // You can change the fixed "4*sigma" region from here.
   xmax = mean + 2.0*sigma;
   func2 = new TF1("func2", "gaus", xmin, xmax);
   energy->Fit("func2", "RQ0");     // Warning!... Do not take option "0" out.
   mean2 = func2->GetParameter(1);
   sigma2 = func2->GetParameter(2);        
   printf("%lf %lf \n", mean2, sigma2);

       if(mean == mean2){
         cout << "\n Quitting Iteration...same mean... \n" << endl;
         break;
       }
   mean = mean2;
   sigma = sigma2;
   }

   cout << "\n xmin = " << xmin << " xmax = " << xmax << "\n" << endl;

   energy->Fit("func2","R");    // This will do the fit in the region between xmin & xmax.. 

  //##############################################
} 


void HecClusterBuildAlg::printTrigger(){
  cout << "Trigger: " ;
  for(int i = 0; i < 5; i++){
    cout << m_event->getHecTriggerBit(i) << " ";
  }
  cout << endl;
}

