
#include "PlotAllCellsAlg.h"
#include "TPostScript.h"
#include "TStyle.h"

#include <sstream>
#include <string>
#include <algorithm>
#include <string>
#include <iostream>

using std::string;
using std::cout;
using std::endl;
using std::ostringstream;



PlotAllCellsAlg::PlotAllCellsAlg(Event* event, Geometry* geo, Headers* runHeader)
  :SystemAlg(event,geo,runHeader){
  m_event = event;
  m_geo = geo;
  m_runHeader = runHeader;
  m_cut = -5;
  m_runHeader->hec()->getLastCh() ;

}


// initialize any variables here
bool PlotAllCellsAlg::initialize(){
  cout << "PlotAllCellsAlg::initialize()\n";
  //this little chunk does the type conversion

  //instaciate the histograms and the canvas for it
  cout << "NumCells: " << m_runHeader->emec()->getLastCh() << endl;

  for(int i =0; i < m_runHeader->emec()->getLastCh() ; i++){
    string runNumber(" Run Number: " );
    std::ostringstream os;
    os  << runNumber << m_runHeader->global()->getRunNo() <<
           "  ADC: " <<  i+1;
    runNumber= os.str();
    string name("");
    std::ostringstream os2;
    os2 << "ADC" << i+1;
    name = os2.str();
    energy.push_back(new TH1F(name.c_str(), runNumber.c_str(), 
    			      100, 0, 0));
    num_entries[i] = 0;
    sum[i] = 0.;
    cell_means.push_back(new ADCMean);
    cell_means[i]->cell = i + 1;
    cell_means[i]->mean = 0.;
    
    // Comment in for automatic rebinning
    //energy[i]->SetBit(TH1::kCanRebin);
  }

 return true;

}

// put any code you want executed per event here
bool PlotAllCellsAlg::execute(){

  for(int i = 0; i < m_runHeader->emec()->getLastCh(); i++){
    //    if(m_event->hecCell(i+1)->signal()>m_cut)  {
    //cout << "Number in signal object" << m_event->emecCell(i)->signal() << endl;
    energy[i]->Fill(m_event->emecCell(i)->signal());
    num_entries[i]++;
    sum[i] = sum[i] + (m_event->emecCell(i)->signal());
    // }
  }

  return true;
}

// Wrap up anything you wanted done after all the events have been 
// run
// ie Draw your histograms etc
bool PlotAllCellsAlg::finalize(){
  cout << "PlotAllCellsAlg::finalize()\n";

  //  TPostScript ps("PlotAllCells.ps", 112);

  TCanvas can("Energy", "Energy Canvas",800,600);

  gStyle->SetOptStat(111111);
  gStyle->SetStatH(0.1);
  gStyle->SetStatW(0.2);

  // the vector is created dynamically because we don't own the 
  // things in it.
  // making this static will cause seg fault.
  vector<TH1F>* histVector = new vector<TH1F>();
  vector<ADCMean>* meanVector = new vector<ADCMean>();

  ostringstream os;
  os << m_runHeader->global()->getRunNo() << "_PlotAll.root";
  TFile file(os.str().c_str(), "recreate");
  
  for(int i = 0; i < m_runHeader->emec()->getLastCh() ; i++) {
    if(num_entries[i]){
      cell_means[i]->mean = sum[i] / (1.0 * num_entries[i]);
    }
    //energy[i]->Fit("gaus");
    histVector->push_back(*energy[i]);
    meanVector->push_back(*cell_means[i]);
    energy[i]->Write();
    // comment the next line out if no postscript output required
    //energy[i]->Draw();
   // can.Update();
  }

  file.Close();
  
  //sort(histVector->begin(),histVector->end(),HistoHighMean());
  sort(meanVector->begin(),meanVector->end(),MeanSort());

  map<int,float> plot;
  for(int i = 0; i < m_runHeader->emec()->getLastCh(); i++){
    if(m_geo->emec_isConnected(i)){
      plot[i] = energy[i]->GetMean();
    }
  }
  m_geo->draw2DEmecDetail(plot,0,false,0,0,"Blah");
  m_geo->draw2DEmec(plot,0,false,0,0,"Blah");

  map<int,float> hecPlot;

   hecPlot[361] = 1.0;
  hecPlot[55] =2.0;
  hecPlot[50] =3.0;
  //m_geo->draw2DEmecDetail(plot);
  m_geo->draw2DHec(hecPlot,0,false,0,0,"Blah");
    m_geo->draw2DHecDetail(hecPlot,0,false,0,0,"Blah");


  // comment in to print the vector
  vector<ADCMean>::iterator itr2;
  for(itr2=meanVector->begin(); itr2!=meanVector->end(); itr2++){
    cout << "ADC" << itr2->cell;
    if(itr2->cell < 10) cout << "    ";
    if(9 < itr2->cell && itr2->cell < 100) cout << "   ";
    if(99 < itr2->cell) cout << "  ";
    cout<< "Mean is: " << itr2->mean << endl;
  }



  //  ps.Close();

  // do some memory cleanup
  //  Warning... if you mess with this I gurantee a segfault.
  vector<TH1F*>::iterator itr3;
  for(itr3 = energy.begin(); itr3 != energy.end(); itr3++){
    delete (*itr3);
  }
  
  vector<ADCMean*>::iterator itr4;
  for(itr4 = cell_means.begin(); itr4 != cell_means.end(); itr4++){
    delete (*itr4);
  }

  return true;
}


